
open Format
open Lexing
open Base
open Ast
open Pretty_print

(* Noms du fichier source *)
let ifile = ref ""

let usage =
  sprintf "usage: %s file.exp" Sys.argv.(0)

(* localise une erreur en indiquant la ligne et la colonne *)
let localisation pos = assert false

let localisation2 (pos1,pos2) = assert false

let () =
  (* Parsing de la ligne de commande *)
  Arg.parse [] (fun s -> ifile := s) usage ;

  (* On vrifie que le nom du fichier source a bien t indiqu *)
  if !ifile="" then begin
    eprintf "Reading on standard input: type in a program...@." ;
    ifile := "/dev/stdin"
  end else
    (* Ce fichier doit avoir l'extension .exp *)
    if not (Filename.check_suffix !ifile ".exp") then begin
      eprintf "Le fichier d'entre doit avoir l'extension .exp\n@?";
      Arg.usage [] usage;
      exit 1
    end ;

  (* Ouverture du fichier source en lecture *)
  let f = open_in !ifile in

  (* Cration d'un tampon d'analyse lexicale *)
  let buf = Lexing.from_channel f in

  try
    (* Parsing: la fonction  Parser.prog transforme le tampon lexical en un
       arbre de syntaxe abstraite si aucune erreur (lexicale ou syntaxique)
       n'est dtecte.
       La fonction Lexer.token est utilise par Parser.prog pour obtenir
       le prochain token. *)
    let p = Parser.prog Lexer.token buf in
    close_in f;

    (* Analyse *)
    Typing.check_program p ;

    printf "OK:@.@[<2>%a@]@." print_prog p

  with
    | Lexer.Lexing_error c ->
	(* Erreur lexicale. On rcupre sa position absolue et
	 * on la convertit en numro de ligne *)
	localisation (Lexing.lexeme_start_p buf);
	eprintf "Erreur dans l'analyse lexicale: %c@." c;
	exit 1
    | Typing.TypeError (s,loc)->
	(* Erreur de typage *)
        localisation2 loc;
        eprintf  "Erreur de typage: %s@." s;
        exit 1
    | Parsing.Parse_error ->
	(* Erreur syntaxique. On rcupre sa position absolue et on la
	 * convertit en numro de ligne *)
	localisation (Lexing.lexeme_start_p buf);
	eprintf "Erreur dans l'analyse syntaxique@.";
	exit 1
