
/* Analyseur syntaxique */

%{

  open Base
  open Ast

%}

%token <int> CST
%token <string> IDENT
%token <Base.op> REL
%token LET IN IF THEN ELSE
%token LP RP VIRG PV DP
%token INT BOOL
%token TRUE FALSE
%token PLUS MINUS TIMES DIV INF INFEQ SUP SUPEQ
%token EOF
%token EQ

/* Dfinitions des priorits et associativits des tokens */

%nonassoc IN
%nonassoc ELSE
%nonassoc REL EQ
%left PLUS MINUS
%left TIMES DIV

/* Point d'entre de la grammaire */
%start prog

/* Type des valeurs retournes par l'analyseur syntaxique */
%type <Ast.program> prog

%%

prog:
| decls EOF { List.rev $1 }
;

typ:
| INT           {TypInt}
| BOOL          {TypBool}
;

var:
| IDENT DP typ  {($1,$3)}
;

lvars:
|                     {[]}
| lvars1              {List.rev $1}
;

lvars1:
| var                 {[$1]}
| lvars1 VIRG var     {$3::$1}

decls:
|  decl               { [$1] }
|  decls PV decl      { $3 :: $1 }
|  decls decl         { $2 :: $1 }
;

decl:
| LET IDENT EQ expr                       { Vdecl ($2, $4) }
| LET IDENT LP lvars RP DP typ EQ expr    { Fdecl ($2,$4,$7,$9) }
;

expr:
| CST                             { Cst (Int $1) }
| TRUE                            { Cst (Bool true) }
| FALSE                           { Cst (Bool false) }
| IDENT                           { Var $1 }
| expr PLUS expr                  { Binop (Plus, $1, $3) }
| expr MINUS expr                 { Binop (Minus, $1, $3) }
| expr TIMES expr                 { Binop (Mult, $1, $3) }
| expr DIV expr                   { Binop (Div, $1, $3) }
| expr REL expr                   { Binop ($2, $1, $3) }
| expr EQ expr                    { Binop (Eq, $1, $3) }
| LET IDENT EQ expr IN expr       { Letin ($2, $4,$6) }
| IF expr THEN expr ELSE expr     { If($2, $4,$6) }
| IDENT LP exprs RP               { Call($1,$3) }
| LP expr RP                      { $2 }
;

exprs:
|                  {[]}
| exprs1           {List.rev $1}
;
exprs1:
| expr             {[$1]}
| exprs1 VIRG expr {$3::$1}
;
